/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.internal.ReportsController");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

(function() { // module container

var checkNotNull = net.jazz.ajax.toolbox.lang.checkNotNull;

var session	= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

/**
 * Main application controller for Reports perspective.
 */
dojo.declare("com.ibm.team.reports.web.app.internal.ReportsController", null, {

	/**
	 * Internal constructor. To be called only from ReportsView.
	 */
	constructor: function(reportsView) {
		messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
		this._view = reportsView;
		this._teamAreas = [];
		this._selectedQuery = null;
		
		this._lastReportArgs = null;
		this._lastQueryArgs = null;
		
		this._includeArchived = false;
		
		this._defaultQuery = null;
	},
	
	/**
	 * Called by Jazz web framework to provide default view when the URL
	 * contains no 'action' request parameter.
	 */
	init: function() {
	},
	
	getSafeUrlActions: function() {
		return ["viewQuery", "viewReport", "getReports", "getSharedQueries", "admin", "refreshNavigator", "newReport"];
	},

	// -------------- action API methods ----------------------------------------------------------
	
	/**
	 * Display the default report, if available.
	 */
	defaultAction: function() {
		if (session.isGuest()) {
			this._view.showWelcome();
			return;
		}
		var responseHandler = new ServiceResponseHandler(
			this,
			"_handleDefaultReport",
			"_remoteError");
		ReportsClient.getDefaultQueryDescriptor(responseHandler);
	},
	
	/**
	 * Run a query for all reports and folders.
	 */
	getReports: function() {
		this._doGetMyReports();
		this._doGetSharedRepors();
	},
	
	/**
	 * View a report query.
	 */
	viewQuery: function(args) {
		checkNotNull(args.queryUUID, "Query id");
		this._selectedQuery = args.queryUUID;
		this._view.showQuery(args);
		this._lastQueryArgs = args;
		this._lastReportArgs = null;
	},
	
	/**
	 * View a report.
	 */
	viewReport: function(args) {
		checkNotNull(args.reportUUID, "Report id");
		this._selectedQuery = null;
		this._view.showReport(args);
		this._lastQueryArgs = null;
		this._lastReportArgs = args;
	},
	
	/**
	 * Administer the data warehouse.
	 */
	admin: function() {
		this._view.admin();
	},
	
	/**
	 * Refresh the left navigator.
	 */
	refreshNavigator: function() {
		this._doGetMyReports();
		this._doGetSharedReports();
	},
	
	/**
	 * Return the UUID of the selected query or
	 * null if no query is selected.
	 */
	getSelectedQuery: function() {
		return this._selectedQuery;
	},
	
	viewWorkItems: function(args) {
		this._view.viewWorkItems(args);
	},
	
	/**
	 * Return whether archived things should be
	 * included in reports
	 */
	getIncludeArchived: function() {
		return this._includeArchived;
	},
	
	/**
	 * Set whether archived things should be
	 * included in reports
	 */
	setIncludeArchived: function(includeArchived) {
		this._includeArchived = includeArchived;
	},
	
	setDefaultQuery: function(defaultQuery) {
		this._defaultQuery = defaultQuery;
		// Notify of the change
		if (this._myReportsNavigatorWidget != null) {
			this._myReportsNavigatorWidget.setDefaultQuery(this._defaultQuery);
		}
		if (this._sharedReportsNavigatorWidget != null) {
			this._sharedReportsNavigatorWidget.setDefaultQuery(this._defaultQuery);
		}
	},
	
	getDefaultQuery: function() {
		return this._defaultQuery;
	},
	
	/**
	 * Marks the start of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	startOperation: function(message) {
		PlatformUI.getWorkbench().showMessage(message);
	},

	/**
	 * Marks the end of a long-running operation (needs more documentation).
	 * May be called by code within the com.ibm.team.workitem.web bundle.
	 */
	finishOperation: function(operation) {
		PlatformUI.getWorkbench().hideMessageArea();
	},
	
	/**
	 * Reload the currently-displayed query or report.
	 */
	reload: function() {
		if (this._lastQueryArgs == null && this._lastReportArgs == null) {
			// Nothing to reload
			return;
		}
		var messageString;
		if (this._includeArchived) {
			messageString = messages.reloadToShowArchived;
		} else {
			messageString = messages.reloadToHideArchived;
		}
		var answer = confirm(messageString);
		if (answer) {
			if (this._lastQueryArgs != null) {
				this._view.showQuery(this._lastQueryArgs);
			} else if (this._lastReportArgs != null) {
				this._view.showReport(this._lastReportArgs);
			}
		}
	},
	
	newReport: function() {
		this._view.newReport();
	},
	
	// ------------- initialization methods to be called from ReportsView ----------------------------
	
	initializeMyReportsNavigator: function(reportsNavigatorWidget) {
		checkNotNull(reportsNavigatorWidget, "reportsNavigatorWidget");
		this._myReportsNavigatorWidget = reportsNavigatorWidget;
		this._doGetMyReports();
	},
	
	initializeSharedReportsNavigator: function(reportsNavigatorWidget) {
		checkNotNull(reportsNavigatorWidget, "reportsNavigatorWidget");
		this._sharedReportsNavigatorWidget = reportsNavigatorWidget;
		this._doGetSharedReports();
	},
	
	// --------------- internal helper methods -----------------------------------------------------
	
	_handleDefaultReport: function(result) {
		if (result != null) {
			this.viewQuery(result);
		} else {
			this._view.showWelcome();
		}
	},
	
	_doGetMyReports: function() {
		var responseHandler = new ServiceResponseHandler(
				this,
				"_displayMyReports",
				"_remoteError");
		ReportsClient.getSystemRootFolderDescriptors(responseHandler);
	},
	
	_doGetSharedReports: function() {
		var responseHandler = new ServiceResponseHandler(
				this,
				"_displaySharedReports",
				"_remoteError");
		ReportsClient.getSystemRootFolderDescriptors(responseHandler);
	},
	
	_displayMyReports: function(result) {
		this._myReportsNavigatorWidget.setFolders(result);
	},
	
	_displaySharedReports: function(result) {
		this._sharedReportsNavigatorWidget.setFolders(result);
	},
	
	_remoteError: function(error) {
		alert("Error invoking service; message: " + error.message);
	}
});

})();